package pong.model;


import pong.service.EventService;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

/*
 * Logic for the Pong Game
 */
public class Pong {

    // Global constants
    public static final double WIDTH = 600;     // Size of window
    public static final double HEIGHT = 400;
    public static final double PADDLE_WIDTH = 10;
    public static final double PADDLE_HEIGHT = 60;
    public static final double BALL_DIAM = 40;

    private final Paddle lPaddle;
    private final Paddle rPaddle;
    private final List<AbstractMovablePongObject> movables = new ArrayList<>();
    private final List<AbstractPongObject> statics = new ArrayList<>();
    private int pointsLeft;
    private int pointsRight;
    private boolean newBall;

    public Pong(Paddle lPaddle, Paddle rPaddle, Ball ball, List<Wall> walls, List<Floor> floors) {
        this.lPaddle = lPaddle;
        this.rPaddle = rPaddle;
        movables.add(ball);
        movables.add(rPaddle);
        movables.add(lPaddle);
        statics.addAll(walls);
        statics.addAll(floors);
    }

    // --------  Game Logic -------------

    // Called by Animation timer about 60 times/sec
    public void update() {

        for (AbstractMovablePongObject o : movables) {
            o.move();
        }

        Iterator<AbstractMovablePongObject> iter = movables.iterator();
        while (iter.hasNext()) {
            AbstractMovablePongObject o = iter.next();
            if (o instanceof Ball) {
                AbstractPongObject theObject = check(statics, o);
                if (theObject instanceof Floor) {
                    ((Ball) o).bounceY();
                } else if (theObject instanceof Wall) {
                    iter.remove();
                    newBall = true;
                }
                theObject = check(movables, o);
                if (theObject instanceof Paddle) {   // null gives false
                    ((Ball) o).bounceX();
                    EventService.add(new EventService.PongEvent(EventService.Type.HIT_PADDLE, null));
                }
            }
        }
        if (newBall) {
            movables.add(new Ball());
            newBall = false;
        }
    }

    // Helper method
    // NOTE : You shouldn't need "? extends ..." anywhere in lab 4. This is just
    // because I'm trapped (... and don't want to write more code to fix it)
    // ? extends AbstractPongObject = some class extending AbstractPong
    private AbstractPongObject check(List<? extends AbstractPongObject> objs, AbstractPongObject theObject) {
        for (AbstractPongObject o : objs) {
            if (o != null && o != theObject && o.intersects(theObject)) {
                return o;
            }
        }
        return null;
    }

    // --- Used at key events /GUI) ------------------------

    public IMovable getLeftPaddle() {
        return lPaddle;
    }

    public IMovable getRightPaddle() {
        return rPaddle;
    }

    // --- Used by GUI (rendering) ------------------------

    private final List<IDrawable> tmp = new ArrayList<>();
    public List<IDrawable> getDrawables() {
        tmp.clear();
        tmp.addAll(statics);
        tmp.addAll(movables);
        //Also possible useful Collections.unmodifiableList(... ));
        return tmp;
    }

    public int getPointsLeft() {
        return pointsLeft;
    }

    public int getPointsRight() {
        return pointsRight;
    }

}
