package pong.service;


import java.util.ArrayList;
import java.util.List;

/*
     Service to handle events **from model** to GUI
     Some data but only used internally

     NOTE: Events from GUI to model handled by JavaFX event
 */
public class EventService {

    public enum Type {
        NONE,
        HIT_PADDLE,
        BALL_OUT,
        HIT_WALL
    }

    // Class for event objects (objects created in model).
    // This is an inner class because it belong very close to EventService
    // Don't need to fully understand
    public static class PongEvent {

        public final Type type;
        public final Object data;  // TODO not used

        public PongEvent(Type type, Object data) {
            this.type = type;
            this.data = data;
        }
    }

    // A default object (if no other event return this)
    private static final PongEvent none = new EventService.PongEvent(Type.NONE, null);
    // Store all incoming events
    private static final List<PongEvent> events = new ArrayList<>();

    public static void add(PongEvent evt) {
        events.add(evt);
    }

    // Remove and return event (called in AnimationTimer loop.
    public static PongEvent remove() {
        if (events.size() > 0) {
            return events.remove(0);
        } else {
            return none;
        }
    }

}
