package pong.view;

import javafx.scene.image.Image;
import javafx.scene.media.AudioClip;
import javafx.scene.paint.Color;
import pong.model.IDrawable;
import pong.service.EventService;
import pong.service.ImageService;
import pong.service.SoundService;

import java.util.HashMap;
import java.util.Map;

import static pong.model.Pong.*;
import static pong.service.EventService.Type;
/*
   Class to hold an actual theme (images and sounds)

   *** Nothing to do here ***
*/


public class Theme {

    public enum Name {
        COOL,
        DUCKIE
    }

    private static Theme actual;


    public static Image getImage(IDrawable d) {
        // NOTE: Name of image file must match this
        String name = d.getClass().getSimpleName().toLowerCase() + d.discriminator() + ".png";
        Image i = actual.nameImage.get(name);
        return i;
    }

    public static Image getBackground() {
        return actual.backgroundImg;
    }

    public static Color getTextColor() {
        return actual.textColor;
    }

    public static void playSound(Type type) {
        actual.sounds.get(type).play();
    }


    public static void playMusic(double vol) {
        actual.music.play(vol);
    }

    public static void stopMusic() {
        actual.music.stop();
    }

    public static boolean isPlayingMusic() {
        return actual.music.isPlaying();
    }

    public static void setTheme(Name themeName) {
        switch (themeName) {
            case COOL:
                actual = new Theme("../assets/img/cool/",
                        Color.WHITE,
                        "ball.png",
                        "paddleblue.png",
                        "paddlered.png",
                        "background.png",
                        "coolMusic.wav",
                        "coolBounce.wav",
                        "",
                        "");
                break;
            case DUCKIE:
                actual = new Theme("../assets/img/duckie/",
                        Color.WHITE,
                        "ball.png",
                        "paddleblue.png",
                        "paddlered.png",
                        "background.jpg",
                        "duckieMusic.wav",
                        "coolBounce.wav",
                        "",
                        "");
                break;
            default:
                new IllegalArgumentException("No such theme");
        }
    }

    public final String imgDir;
    public final Color textColor;
    public final Image backgroundImg;
    Map<String, Image> nameImage = new HashMap<>();
    public final Map<EventService.Type, AudioClip> sounds = new HashMap<>();
    public final AudioClip music;

    private Theme(String dir,
                  Color textColor,
                  String ballImgName,
                  String paddleRightImgName,
                  String paddleLeftImgName,
                  String backgroundImgName,
                  String music,
                  String hitPaddleSound,
                  String hitWallSound,
                  String ballOutSound) {
        imgDir = dir;
        this.textColor = textColor;
        nameImage.put(ballImgName, ImageService.getImage(imgDir, ballImgName, BALL_DIAM, BALL_DIAM));
        nameImage.put(paddleRightImgName, ImageService.getImage(imgDir, paddleRightImgName, PADDLE_WIDTH, PADDLE_HEIGHT, false));
        nameImage.put(paddleLeftImgName, ImageService.getImage(imgDir, paddleLeftImgName, PADDLE_WIDTH, PADDLE_HEIGHT, false));
        backgroundImg = ImageService.getImage(imgDir, backgroundImgName, WIDTH, HEIGHT, false);
        this.music = SoundService.getMusic(music);
        sounds.put(Type.HIT_PADDLE, SoundService.getSound(hitPaddleSound));
        //sounds.put(Type.HIT_WALL, SoundService.getSound(hitWallSound));
        //sounds.put(Type.BALL_OUT, SoundService.getSound(ballOutSound));
    }

}
