package samples;

import java.util.*;

import static java.lang.System.out;

/**
 * Using a Class of our own,  here we create a class for Complex numbers
 */
public class UseAClass {


    public static void main(String[] args) {
        new UseAClass().program();
    }

    void program() {
        // Create objects of type Complex
        Complex c1 = new Complex(1, 2);   // Instantiate, use constructor on right side
        Complex c2 = new Complex(-3, -5);
        Complex c3 = new Complex(c2);     // Use constructor to create a copt

        out.println(c1.getRe() == 1);    // Call methods
        out.println(c1.getImg() == 2);

        out.println(c3.equals(c2));
        out.println(c1.sub(c2).equals(new Complex(4, 7)));
        out.println(c1.add(c1).add(c1).equals(new Complex(3, 6)));  // Chained call

        Complex[] cArr = {new Complex(1, 2), new Complex(6, -1)};    // Array of own objects
        out.println(cArr[0].sub(cArr[0]).equals(new Complex(0, 0)));

        List<Complex> cList = new ArrayList<>();   // List of own objects
        cList.add(new Complex(5, -2));
        out.println(cList.contains(new Complex(5, -2))); // Must have equals()

        Map<Complex, String> complexMsg = new HashMap<>();
        complexMsg.put(new Complex(0,0), "zero");               // Must have hashCode()
        out.println(complexMsg.get(new Complex(0,0)).equals("zero"));

        out.println(new Complex(5, -2).toString().equals("5.0 - 2.0i"));
    }


    // -------- Class for complex numbers (not finished) ----------

    class Complex {      // Class declaration
        final double re; // Instance variables, class is immutable!
        final double img;

        Complex(double re, double img) {  // Constructor, called att instantiation with new
            this.re = re;
            this.img = img;
        }

        Complex(Complex other) {   // Another constructor, a copy constructor
            re = other.re;
            img = other.img;
        }

        // --- Methods ---------------

        // Always return a new Complex (because we normally assume operands won't change)
        Complex add(Complex other) {
            return new Complex(this.re + other.re, this.img + other.img);
        }

        Complex sub(Complex other) {
            return new Complex(this.re - other.re, this.img - other.img);
        }

        double getRe() {    // Possible access parts if complex
            return re;
        }

        double getImg() {
            return img;
        }

        /*
            Below are 3 method that your subclasses probably should implement

            The methods are in fact present in any class, inherited from the class Object, but the
            inherited method don't behave as we normally wish, so usually need to create own methods.
            Because of inheritance all methods should have @Override and public, more later

            NOTE : All of them can be Generated by IntelliJ, right click > Generate ...

         */
        // This is the way to implement equality (by value)
        // If not adding this method you'll get equality by reference (i.e. identity)
        // If not adding this method objects in Collections will not be found!
        @Override
        public boolean equals(Object o) {
            if (this == o) {
                return true;
            }
            if (o == null || getClass() != o.getClass()) {
                return false;
            }
            Complex complex = (Complex) o;

            return Double.compare(complex.re, re) == 0 &&
                    Double.compare(complex.img, img) == 0;

        }
        // This is used by Map's.
        // If not implemented objects in Maps will not be found if
        // this is used as key
        @Override
        public int hashCode() {
            int result;
            long temp;
            temp = Double.doubleToLongBits(re);
            result = (int) (temp ^ (temp >>> 32));
            temp = Double.doubleToLongBits(img);
            result = 31 * result + (int) (temp ^ (temp >>> 32));
            return result;
        }

        // Create a human readable String out of an object (or parts of object)
        // This method automatically called at out.println()
        // Used for debug.
        @Override
        public String toString() {
            if (re == 0 && img == 0) {
                return "0";
            } else if (re == 0) {
                return img + "i";
            } else if (img == 0) {
                return Double.toString(re);
            } else if (img > 0) {
                return re + " + " + (img + "i");
            } else {
                return re + " - " + (-1 * img + "i");
            }
        }
    }

}
